@@ -1,3 +1,18 @@
+#' Private predicate function that checks if the protocol of a url
+#' is https.
+#'
+#' @param x is a url string
+is.https <- function(x) {
+  m <- regexec("^https?://", x)
+  matches <- regmatches(x, m)
+  if (matches=="https://") {
+    h <- TRUE
+  } else {
+    h <- FALSE
+  }
+  h
+}
+
 #' Private function for performing a GET request
 #'
 #' @param endpoint /path for REST request
@@ -10,9 +25,17 @@ yhat.get <- function(endpoint, query=c()) {
 
   if ("env" %in% names(AUTH)) {
     url <- AUTH[["env"]]
-    url <- stringr::str_replace_all(url, "^http://", "")
+    usetls <- FALSE
+    if (is.https(url)) {
+        usetls <- TRUE
+    }
+    url <- stringr::str_replace_all(url, "^https?://", "")
     url <- stringr::str_replace_all(url, "/$", "")
-    url <- sprintf("http://%s/", url)
+    if (usetls) {
+      url <- sprintf("https://%s/", url)
+    } else {
+      url <- sprintf("http://%s/", url)
+    }
     AUTH <- AUTH[!names(AUTH)=="env"]
     query <- c(query, AUTH)
     query <- paste(names(query), query, collapse="&", sep="=")
@@ -32,12 +55,21 @@ yhat.verify <- function() {
     stop("Please define a yhat.config object")
   })
   env <- AUTH[["env"]]
-  env <- stringr::str_replace_all(env, "^http://", "")
+  usetls <- FALSE
+  if (is.https(env)) {
+    usetls <- TRUE
+  }
+  env <- stringr::str_replace_all(env, "^https?://", "")
   env <- stringr::str_replace_all(env, "/$", "")
   username <- AUTH[["username"]]
   apikey <- AUTH[["apikey"]]
-  url <- sprintf("http://%s/verify?username=%s&apikey=%s",
-                 env, username, apikey)
+  if (usetls) {
+    url <- sprintf("https://%s/verify?username=%s&apikey=%s",
+                   env, username, apikey)
+  } else {
+    url <- sprintf("http://%s/verify?username=%s&apikey=%s",
+                   env, username, apikey)
+  }
   rsp <- httr::POST(url)
   if (httr::http_status(rsp)$category != "success") {
     stop(sprintf("Bad response from http://%s/", env))
@@ -67,9 +99,17 @@ yhat.post <- function(endpoint, query=c(), data, silent = TRUE) {
 
   if ("env" %in% names(AUTH)) {
     url <- AUTH[["env"]]
-    url <- stringr::str_replace_all(url, "^http://", "")
+    usetls <- FALSE
+    if (is.https(url)) {
+      usetls <- TRUE
+    }
+    url <- stringr::str_replace_all(url, "^https?://", "")
     url <- stringr::str_replace_all(url, "/$", "")
-    url <- sprintf("http://%s/", url)
+    if (usetls) { 
+      url <- sprintf("https://%s/", url)
+    } else {
+      url <- sprintf("http://%s/", url)
+    }
     AUTH <- AUTH[!names(AUTH)=="env"]
     query <- c(query, AUTH)
     query <- paste(names(query), query, collapse="&", sep="=")
@@ -137,7 +177,7 @@ check.dependencies <- function() {
 #' yhat.config <- c(
 #'  username = "your username",
 #'  apikey = "your apikey",
-#'  env = "http://cloud.yhathq.com/"
+#'  env = "http://sandbox.yhathq.com/"
 #' )
 #' \dontrun{
 #' yhat.show_models()
@@ -200,10 +240,17 @@ yhat.predict_raw <- function(model_name, data, model_owner, raw_input = FALSE, s
 
   # build the model url for the error message
   url <- AUTH[["env"]]
-  url <- stringr::str_replace_all(url, "^http://", "")
+  usetls <- FALSE
+  if (is.https(url)) {
+    usetls <- TRUE
+  }
+  url <- stringr::str_replace_all(url, "^https?://", "")
   url <- stringr::str_replace_all(url, "/$", "")
-  model_url <- sprintf("http://%s/model/%s/", url, model_name)
-  
+  if (usetls) {
+    model_url <- sprintf("https://%s/model/%s/", url, model_name)
+  } else {
+    model_url <- sprintf("http://%s/model/%s/", url, model_name)
+  }
   query <- list()
   if (raw_input==TRUE) {
     query[["raw_input"]] <- "true"
@@ -242,7 +289,7 @@ yhat.predict_raw <- function(model_name, data, model_owner, raw_input = FALSE, s
 #' yhat.config <- c(
 #'  username = "your username",
 #'  apikey = "your apikey",
-#'  env = "http://cloud.yhathq.com/"
+#'  env = "http://sandbox.yhathq.com/"
 #' )
 #' \dontrun{
 #' yhat.predict("irisModel", iris)
@@ -311,13 +358,14 @@ yhat.test_predict <- function(data, verbose=FALSE) {
 #' via Yhat's REST API (see \code{\link{yhat.predict}}).
 #'
 #' @param model_name name of your model
+#' @param packages list of packages to install using apt-get
 #' @keywords deploy
 #' @export
 #' @examples
 #' yhat.config <- c(
 #'  username = "your username",
 #'  apikey = "your apikey",
-#'  env = "http://cloud.yhathq.com/"
+#'  env = "http://sandbox.yhathq.com/"
 #' )
 #' iris$Sepal.Width_sq <- iris$Sepal.Width^2
 #' fit <- glm(I(Species)=="virginica" ~ ., data=iris)
@@ -336,7 +384,7 @@ yhat.test_predict <- function(data, verbose=FALSE) {
 #' \dontrun{
 #' yhat.deploy("irisModel")
 #' }
-yhat.deploy <- function(model_name) {
+yhat.deploy <- function(model_name, packages=c()) {
   if(missing(model_name)){
     stop("Please specify 'model_name' argument")
   }
@@ -351,12 +399,20 @@ yhat.deploy <- function(model_name) {
   }
   if ("env" %in% names(AUTH)) {
     env <- AUTH[["env"]]
-    env <- stringr::str_replace_all(env, "^http://", "")
+    usetls <- FALSE
+    if (is.https(env)) {
+      usetls <- TRUE
+    }
+    env <- stringr::str_replace_all(env, "^https?://", "")
     env <- stringr::str_replace_all(env, "/$", "")
     AUTH <- AUTH[!names(AUTH)=="env"]
     query <- AUTH
     query <- paste(names(query), query, collapse="&", sep="=")
-    url <- sprintf("http://%s/deployer/model?%s", env, query)
+    if (usetls) {
+      url <- sprintf("https://%s/deployer/model?%s", env, query)
+    } else {
+      url <- sprintf("http://%s/deployer/model?%s", env, query)
+    }
     image_file <- ".yhatdeployment.img"
     all_objects <- yhat.ls()
     all_funcs <- all_objects[lapply(all_objects, function(name){
@@ -377,6 +433,7 @@ yhat.deploy <- function(model_name) {
                            "model_image" = httr::upload_file(image_file),
                            "modelname" = model_name,
                            "packages" = capture.packages(),
+                           "apt_packages" = packages,
 			   "code" = capture.src(all_funcs)
                                  )
                          )
@@ -407,7 +464,7 @@ yhat.deploy <- function(model_name) {
 #' yhat.config <- c(
 #'  username = "your username",
 #'  apikey = "your apikey",
-#'  env = "http://cloud.yhathq.com/"
+#'  env = "http://sandbox.yhathq.com/"
 #' )
 #' iris$Sepal.Width_sq <- iris$Sepal.Width^2
 #' fit <- glm(I(Species)=="virginica" ~ ., data=iris)
@@ -423,7 +480,9 @@ yhat.deploy <- function(model_name) {
 #' model.predict <- function(df) {
 #'  data.frame("prediction"=predict(fit, df, type="response"))
 #' }
-#' yhat.deploy.to.file("irisModel")
+#' \dontrun{
+#'  yhat.deploy.to.file("irisModel")
+#'  }
 yhat.deploy.to.file <- function(model_name) {
   if(missing(model_name)){
     stop("Please specify 'model_name' argument")
@@ -478,7 +537,9 @@ yhat.deploy.to.file <- function(model_name) {
 #' model.predict <- function(df) {
 #'  data.frame("prediction"=predict(fit, df, type="response"))
 #' }
-#' yhat.deploy.with.scp("irisModel", "~/path/to/pemfile.pem")
+#' \dontrun{
+#'  yhat.deploy.with.scp("irisModel", "~/path/to/pemfile.pem")
+#' }
 yhat.deploy.with.scp <- function(model_name, pem_path) {
   yhat.deploy.to.file(model_name)
   filename <- paste(model_name, ".yhat", sep="")
@@ -598,6 +659,10 @@ yhat.transform_from_example <- function(df) {
 #' @param defined.vars variables which have already been defined within the
 #'          scope of the block. e.g. function argument
 yhat.spider.block <- function(block,defined.vars=c()){
+    # if block is a symbol, just return that symbol
+    if(typeof(block) == "symbol") {
+        return(c(block))
+    }
     symbols <- c()
     n <- length(block)
     if(n == 0) {
@@ -679,7 +744,7 @@ yhat.spider.func <- function(func.name){
 #' List all object names which are dependencies of `model.transform`
 #' and `model.predict`
 yhat.ls <- function(){
-    funcs <- c("model.transform","model.predict") # function queue to spider
+    funcs <- c("model.predict", "model.transform") # function queue to spider
     dependencies <- funcs
     global.vars <- ls(.GlobalEnv,all.names=T)
     for (func in funcs){
